/*
 * FILE --- lqr_ctrl.cc
 *
 * PURPOSE --- Source file for a class that implements a State Feddback or
 * Linear Quadratic Regulator (LQR) controller. The user specifies the
 * dimension of the state of the system under control and the LQR gains.
 *
 */

// Include Files
#include "lqr_ctrl.hh"

// Define this macro if you want to use this utility without linking
// to the RCS Library
#ifdef NO_RCS
#include <stdio.h>            /* printf function */
#else
#include "rcs_prnt.hh"          /* rcs_print_ functions */
#endif


/*************************************************************/

LQR_CTRL::LQR_CTRL(int _plant_order)
{
  // No error at start
  lqr_ctrl_err = LQR_CTRL_NO_ERROR;

  // Initialize plant parameters
  plant_order = _plant_order;

  // Allocate memory 
  allocate_memory();
  
  if (lqr_ctrl_err != LQR_CTRL_NO_ERROR)
    {
#ifdef NO_RCS
      printf("LQR_CTRL: ERROR --- Not Enought Memory!!!\n");
      printf("The LQR controller will not function correctly ...\n");
#else
      rcs_print("LQR_CTRL: ERROR --- Not Enought Memory!!!\n");
      rcs_print("The LQR controller will not function correctly ...\n");
#endif
    }

  // Set all the gains to zero
  set_gains();
}

LQR_CTRL::~LQR_CTRL()
{
  // Free the allocated memory
  free_memory();
}

void LQR_CTRL::allocate_memory(void)
{
  if ((lqr_gains = new double[plant_order]) == NULL)
    lqr_ctrl_err = LQR_CTRL_MEM_ERROR;
}

void LQR_CTRL::free_memory(void)
{
  delete [] lqr_gains;
}

double LQR_CTRL::control(const double *_input)
{
  double lqr_output = 0.0;

  // Calculate the controller output
  for (int ii=0; ii < plant_order; ii++)
    {
      lqr_output += lqr_gains[ii] * _input[ii];
    }

  return(lqr_output); 
}


void LQR_CTRL::set_gains(double *_lqr_gains = NULL)
{
  if (_lqr_gains == NULL) // No gains specified
    {
      for (int ii=0; ii < plant_order; ii++)
	{
	  lqr_gains[ii] = 0.0;
	}
    }
  else  // Set the specified gains
    {
      for (int ii=0; ii < plant_order; ii++)
	{
	  lqr_gains[ii] = _lqr_gains[ii];
	}
    }
}
